/************************************************************************
	Mini Button
*************************************************************************/	

function MiniButton(name, hOffset, vOffset, src, parent, action)
{
	this.hOffset = hOffset;
	this.vOffset = vOffset;
	this.src = src;
	this.enabled = true;
	
	this.background = new Image();
	this.background.src = src + ".png";
	this.background.hOffset = hOffset;
	this.background.vOffset = vOffset;
	this.background.tracking = "rectangle";
	this.background.onMouseDown = "if(" + name + ".enabled) " + name + ".setState(false);";
	this.background.onMouseUp = "if(" + name + ".enabled) " + name + ".setState(true);";
	this.background.onClick = "if(" + name + ".enabled) " + action;
	parent.appendChild(this.background);
	
	this.watch("hOffset", MiniButton.prototype.applyProperty);
	this.watch("vOffset", MiniButton.prototype.applyProperty);
	this.watch("opacity", MiniButton.prototype.applyProperty);
	this.watch("tooltip", MiniButton.prototype.applyProperty);
	this.watch("colorize", MiniButton.prototype.applyProperty);
	this.watch("src", MiniButton.prototype.srcWatch);
	
	this.setState(true);
}

MiniButton.prototype.srcWatch = function(prop, oldval, newval)
{
	this.src = newval;
	this.setState(true);
	return newval;
}

MiniButton.prototype.setEnabled = function(enable)
{
	this.background.opacity = enable ? 255 : 135;
	this.enabled = enable;
}

MiniButton.prototype.setState = function(normal)
{
	if(this.src.length > 0)
	{
		this.background.src = this.src + (normal ? ".png" : "Down.png");
		this.width = this.background.width;
	}
}

MiniButton.prototype.applyProperty = function(prop, oldval, newval)
{
	this.background[prop] = newval;
	return newval;
}

/************************************************************************
	Button
*************************************************************************/	

function Button(name, hOffset, vOffset, label, action, parent)
{
	this.enabled = true;

	// Button's background
	this.background = new Image();
	this.background.src = "Resources/Images/button.png";
	this.background.hAlign = "center";
	this.background.hOffset = hOffset;
	this.background.vOffset = vOffset;
	this.background.tracking = "rectangle";
	this.background.onMouseDown = "if(" + name + ".enabled) " + name + ".background.src = 'Resources/Images/buttonDown.png';";
	this.background.onMouseUp = "if(" + name + ".enabled) " + name + ".background.src = 'Resources/Images/button.png';";
	this.background.onClick = "if(" + name + ".enabled) " + action;
	parent.appendChild(this.background);
	
	// Button's label
	this.label = new Text();
	this.label.data = label;
	this.label.hAlign = "center";
	this.label.hOffset = hOffset;
	this.label.vOffset = vOffset + this.background.height - 8;
	this.label.size = 12;
	this.label.style.fontWeight = "bold";
	this.label.onMouseDown = "if(" + name + ".enabled) " + name + ".background.src = 'Resources/Images/buttonDown.png';";
	this.label.onMouseUp = "if(" + name + ".enabled) " + name + ".background.src = 'Resources/Images/button.png';";
	this.label.onClick = "if(" + name + ".enabled) " + action;
	parent.appendChild(this.label);
}

Button.prototype.updateAppearance = function()
{
	this.background.colorize = getImageColor();
	this.label.color = getTextColor();
	this.label.shadow = getShadowColor();
}

Button.prototype.setEnabled = function(enable)
{
	this.label.opacity = this.background.opacity = enable ? 255 : 135;
	this.enabled = enable;
}

/************************************************************************
	Popup
*************************************************************************/	

function Popup(name, hOffset, vOffset, option, optionValue, defaultValue, parent)
{
	this.name = name;
	this.option = option;
	this.optionValue = optionValue;
	this.defaultValue = defaultValue;

	// Popup's background
	this.background = new Image();
	this.background.src = "Resources/Images/popup.png";
	this.background.hOffset = hOffset;
	this.background.vOffset = vOffset;
	this.background.tracking = "rectangle";
	this.background.onMouseDown = name + ".background.src = 'Resources/Images/popupDown.png'";
	this.background.onMouseUp = name + ".background.src = 'Resources/Images/popup.png'";
	this.background.onClick = name + ".showOptions();";
	parent.appendChild(this.background);
	
	// Popup's label
	this.label = new Text();
	this.label.size = 12;
	this.label.style.fontWeight = "bold";
	this.label.hOffset = hOffset + 9;
	this.label.vOffset = vOffset + this.background.height - 8;
	this.label.width = 107;
	this.label.truncation = "end";
	this.label.onMouseDown = name + ".background.src = 'Resources/Images/popupDown.png'";
	this.label.onMouseUp = name + ".background.src = 'Resources/Images/popup.png'";
	this.label.onClick = name + ".showOptions();";
	parent.appendChild(this.label);
}

// Show popup's options menu
Popup.prototype.showOptions = function()
{
	var items = new Array();
	
	// Load popup's options
	for(var i = 0; i < this.option.length; i++)
	{
		items[i] = new MenuItem();
		items[i].title = this.option[i];
		items[i].checked = this.value == this.optionValue[i];
		items[i].onSelect = this.name + ".setValue('" + this.optionValue[i] + "');";
	}
	
	var point = this.background.convertPointToWindow(1, this.background.height - 1);
	popupMenu(items, point.x, point.y);
	this.background.src = "Resources/Images/popup.png";
}

Popup.prototype.setValue = function(value)
{
	// Find the label which is associated with the chosen value
	for(var ind in this.optionValue)
	{
		if(this.optionValue[ind] == value)
		{
			this.label.data = this.option[ind];
			this.value = value;
			return;
		}
	}
	
	// If no label was found, use default value
	this.setValue(this.defaultValue);
}

Popup.prototype.updateAppearance = function()
{
	this.background.colorize = getImageColor();
	this.label.color = getTextColor();
	this.label.shadow = getShadowColor();
}

/************************************************************************
	Throbber Object
	===============
	
	Activity indicator.
*************************************************************************/	

function Throbber(parent)
{
	this.on = false;
	this.frame = 0;
	this.color = "255, 255, 255";

	this.canvas = new Canvas();
	this.canvas.hAlign = "center";
	this.canvas.vAlign = "center";
	this.canvas.width = 110;
	this.canvas.height = 110;
	parent.appendChild(this.canvas);
	
	this.ctx = this.canvas.getContext("2d");

	this.timer = new Timer();
	this.timer.interval = 0.05;
	this.timer.throbber = this;
	this.timer.onTimerFired = Throbber.prototype.update;
	this.timer.ticking = false;
	
	this.watch("hOffset", Throbber.prototype.applyProperty);
	this.watch("vOffset", Throbber.prototype.applyProperty);
	this.watch("opacity", Throbber.prototype.applyProperty);
	this.watch("size", Throbber.prototype.applySizeProperty);
	
	this.fadeIn = new FadeAnimation(this.canvas, 128, 500, animator.kEaseOut);
	this.fadeOut = new FadeAnimation(this.canvas, 0, 500, animator.kEaseIn);
}

Throbber.prototype.applyProperty = function(prop, oldval, newval)
{
	this.canvas[prop] = newval;
	return newval;
}

Throbber.prototype.applySizeProperty = function(prop, oldval, newval)
{
	this.canvas.width = this.canvas.height = newval;
	this.sizePortion = newval/110;
	return newval;
}

Throbber.prototype.start = function(animate)
{
	this.frame = 0; 
	this.on = this.timer.ticking = true;
	
	if(animate)
	{
		this.fadeOut.kill();
		animator.start(this.fadeIn);
	}
	else this.opacity = 128;
}

Throbber.prototype.stop = function(animate)
{
	this.on = this.timer.ticking = false;
	
	if(animate)
	{
		this.fadeIn.kill();
		animator.start(this.fadeOut);
	}
	else this.opacity = 0;
}

Throbber.prototype.update = function()
{
	var _throbber = this.throbber;

	_throbber.ctx.clearRect(0, 0, _throbber.canvas.width, _throbber.canvas.height);
	
	for(var i = 0; i < 13; i++)
	{
		_throbber.ctx.beginPath();
		_throbber.ctx.arc(55*_throbber.sizePortion + 48*_throbber.sizePortion*Math.cos(360/13*(i + _throbber.frame)*Math.PI/180), 55*_throbber.sizePortion + 48*_throbber.sizePortion*Math.sin(360/13*(i + _throbber.frame)*Math.PI/180), 7*_throbber.sizePortion, 0, Math.PI*2, false);
		_throbber.ctx.fillStyle = "rgba(" + _throbber.color + ", " + 1/13*(i + 1) + ")";
		_throbber.ctx.fill();
	}
	
	_throbber.frame++;
	if(_throbber.frame == 13) _throbber.frame = 0;
}