﻿/************************************************************************
	Feed Object
*************************************************************************/

function Feed(settings)
{
	// Apply default settings
	this.title = "";
	this.userTitle = "";
	this.url = "";
	this.updateFrequency = DEFAULT_UPDATE_FREQUENCY;
	this.maxVisibleItems = DEFAULT_MAX_FEED_ITEMS;
	this.readItems = 0;
	this.error = false;
	this.safeMode = false;
	this.items = new Array();

	// Apply initial settings
	for(var ind in settings)
	{
		this[ind] = settings[ind];
	}
	
	this.safeMode = parseBool(this.safeMode);

	// Feed's refresh timer
	this.refreshTimer = new Timer();
	this.refreshTimer.interval = this.updateFrequency*60;
	this.refreshTimer.feed = this;
	this.refreshTimer.onTimerFired = function(){this.feed.refresh();}
	this.refreshTimer.ticking = true;

	// Create the feed's XML HTTP request object
	this.request = new XMLHttpRequest();
	this.request.autoRedirect = true;
	this.request.feed = this;
}


Feed.prototype.refresh = function()
{
	if(this.refreshing) return; // No need to refresh a feed if it is already refreshing
	
	// Reset settings
	this.refreshing = true;
	this.error = false;
	this.row.setTitle(this.userTitle ? this.userTitle : (this.title ? this.title : loc("loading")));
	this.row.setActive(true, widgetInitialized);
	this.readItems = 0;
	if(!overlay.opacity && preferences.descriptionsType.value == "0") this.row.setTooltip(this.getDescription());

	try
	{
		if(feedsPane.selectedRow == this.row)
		{
			UpdateFeedTitle();
			UpdateUpdatedText();
			contentPane.setListState(false, true, true);
			if(contentPane.rows.length > 0) contentPane.clearList();
		}

		// send request
		this.request.onreadystatechange = this.onFeedFetchDone;
		this.request.url = this.url;
		this.request.open("GET", this.url, true);
		this.request.send(); // Start loading data
	}
	catch(e)
	{
		this.onRefreshError("refresh", "An exception occurred while trying to refresh the RSS feed \"" + this.url + "\".", e);
	}
}

Feed.prototype.onFeedFetchDone = function()
{
	if(this.readyState == 4)
	{
		switch(this.status)
		{
			case 200:
				this.feed.onRefreshCompleted(this.responseText);
			break
			case 304:
				this.updateDisplay();
			break;
			default:
				this.feed.onRefreshError("onFeedFetchDone", "An HTTP error \"" + this.responseText + "\" (code: " + this.status + ") occurred while trying to fetch \"" + this.url + "\".");
			break
		}
	}
}

// Load the feed from the specified XML text
Feed.prototype.onRefreshCompleted = function(xml)
{
	allFeeds.feedUpdated = this.feedUpdated = new Date();
	
	try
	{
		this.items.splice(0, this.items.length);
	
		/*
		  Invalid feeds might crash the Widget once it attempts to parse them so it's needed
		  to keep track of them before and after they are being loaded. If the Widget crashes, the URL
		  of the feed which caused the crash should appear in the log text file.
		*/
	
		filesystem.writeFile(system.widgetDataFolder + "/log.txt",  this.url);
	
		var doc;
	
		// Start parsing data
		if(this.safeMode) doc = XMLDOM.parse(convertHtmlEntities(xml, "noXmlEntities"));
		else	doc = XMLDOM.parse(xml);
	
		filesystem.writeFile(system.widgetDataFolder + "/log.txt",  "");
		
		if (doc != null) {
			var itemsRootElement;
			
			// Recognize feed type:
			var docRootElement = doc.evaluate("rss");
			if (docRootElement.length > 0) {
				// RSS Version 2.0
				this.type = "rss";
				docRootElement = docRootElement.item(0);
				itemsRootElement = docRootElement.evaluate("channel").item(0);
			} else {
				docRootElement = doc.evaluate("feed");
				if (docRootElement.length > 0) {
					// Atom
					this.type = "atom";
					docRootElement = docRootElement.item(0);
					itemsRootElement = docRootElement;
				} else {
					docRootElement = doc.evaluate("rdf:RDF");
					if (docRootElement.length > 0) {
						// RSS Version 1.0
						this.type = "rss";
						docRootElement = docRootElement.item(0);
						itemsRootElement = docRootElement;
					}	else {
						this.onRefreshError("onRefreshCompleted", "Unrecognized feed.");
					}
				}
			}

			if (docRootElement && itemsRootElement) { // Make sure the root element was found
				// Parse feed information
				this.title = convertHtmlEntities(docRootElement.evaluate("string(" + this.getElementPath("feedTitle") + ")"), this.safeMode ? "onlyXmlEntities" : null);
				this.link = docRootElement.evaluate("string(" + this.getElementPath("feedLink") + ")");
				this.description = convertHtmlEntities(getShortString(docRootElement.evaluate("string(" + this.getElementPath("feedDescription") + ")"), 190), this.safeMode ? "onlyXmlEntities" : null);
				
				var itemElements = itemsRootElement.evaluate(this.getElementPath("item"));

				// Load items
				for(var itemIndex = 0; itemIndex < itemElements.length; itemIndex++)
				{
					var itemElement = itemElements.item(itemIndex);
					var strDate = '';

					var feedItem = new FeedItem(this, itemIndex);
					feedItem.title = convertHtmlEntities(itemElement.evaluate("string(" + this.getElementPath("itemTitle") + ")"), this.safeMode ? "onlyXmlEntities" : null);
					feedItem.link = itemElement.evaluate("string(" + this.getElementPath("itemLink") + ")");
					feedItem.description = convertHtmlEntities(itemElement.evaluate("string(" + this.getElementPath("itemDescription") + ")").replace(/<[^>]+>/g, " "), this.safeMode ? "onlyXmlEntities" : null);
					feedItem.guid = itemElement.evaluate("string(" + this.getElementPath("itemGuid") + ")");
					strDate = itemElement.evaluate("string(" + this.getElementPath("itemPubDate") + ")");
					if (strDate) {
						if(this.type == "rss") feedItem.pubDate = new Date(strDate);
						if(this.type == "atom") feedItem.pubDate = Feed.dateFromRFC3339(strDate);
					}
					else
					{
						if(this.type == "rss")
						{
							strDate = itemElement.evaluate("string(dc:date)");
							feedItem.pubDate = Feed.dateFromRFC3339(strDate);
						}
						
						if(this.type == "atom")
						{
							strDate = itemElement.evaluate("string(modified)");
							if(!strDate) strDate = itemElement.evaluate("string(created)");
							feedItem.pubDate = Feed.dateFromRFC3339(strDate);
						}
					}
					
					if (!feedItem.pubDate || isNaN(feedItem.pubDate.valueOf())) {
						// if no date is given/recognized, give it a "0" date to put it at the bottom of the list
						feedItem.pubDate = new Date(0);
					}

					this.items.push(feedItem);
					
					// Check if this item has already been read
					var id = feedItem.guid ? feedItem.guid : feedItem.link;
					if(getArrayElementIndex(readItems, id) != -1)
					{
						feedItem.read = true;
						this.readItems += 1;
					}
					else feedItem.read = false;

					if(this.items.length >= this.maxVisibleItems) break;
				}
				this.items.sort(Feed.sortFunc);
				this.refreshing = false;

				this.updateDisplay();
			}
			else this.onRefreshError("onRefreshCompleted", "ERROR: empty 'Document Root' element.");
		}
		else this.onRefreshError("onRefreshCompleted", "ERROR: invalid XML document.");
	}
	catch(e)
	{
		this.onRefreshError("onRefreshCompleted", "An exception occurred while trying to parse the RSS feed \"" + this.url + "\".", e);
	}
	
	allFeeds.refresh();
	allFeeds.updateDisplay();
}

Feed.prototype.onRefreshError = function(func, errorMessage, e)
{
	printError(func, errorMessage, e)

	this.feedUpdated = null;
	this.refreshing = false;
	this.description = loc("couldNotLoadFeedInfo");
	this.error = true;
	
	this.updateDisplay();
	
	allFeeds.refresh();
	allFeeds.updateDisplay();
}

// Update the Widget's display with the feed's data
Feed.prototype.updateDisplay = function()
{
	// Update Widget's display
	
	if (!this.error) {
		this.row.setTitle(this.getTitle());
		this.row.setActive(this.refreshing || (this.readItems < this.items.length), widgetInitialized);
	} else {
		this.row.setTitle(this.description);
		this.row.setActive(false, widgetInitialized);
	}
	
	if(!overlay.opacity && preferences.descriptionsType.value == "0") this.row.setTooltip(this.description);
	if(fsFrame && sheetFrame.sheet == fsFrame) UpdateFSTitle();
	if(feedsPane.selectedRow == this.row) ShowFeed(this);
	
	if(tRow == this.row)
	{
		if(preferences.descriptionsType.value == "1")
		{
			UpdateTooltip(this.getTitle(true), this.description, this.feedUpdated ? timeToString(this.feedUpdated.getTime()) : "");
		}
		
		if(preferences.descriptionsType.value == "2")
		{
			UpdateTray(this.getTitle(true), this.description, this.feedUpdated ? timeToString(this.feedUpdated.getTime()) : "");
		}

	}
}

// @param orDomain - if the title is empty return the domain URL
// @param neverEmpty - never return an empty string

Feed.prototype.getTitle = function(orDomain, neverEmpty)
{
	var title = this.userTitle ? this.userTitle : this.title;
	
	if(orDomain)
	{
		if(title) return title;
		else
		{
			var url = this.url.match(/^(.*:\/\/)?(.*)/)[2];
			return url ? url : this.url;
		}
	}
	else if((!title || title.length == 0) && neverEmpty) return loc("untitled");
	else return title;
}

Feed.prototype.getDescription = function()
{
	return this.refreshing && !this.description ? loc("loading") : this.description;
}

// Return the path to s specified element according to the feed type
Feed.prototype.getElementPath = function(element)
{
	switch(element)
	{
		case "feedTitle": return this.type == "rss" ? "channel/title" : "title";  break;
		case "feedLink": return this.type == "rss" ? "channel/link" : "link/attribute::href";  break;
		case "feedDescription": return this.type == "rss" ? "channel/description" : "tagline"; break;
		case "item": return this.type == "rss" ? "item" : "entry"; break;
		case "itemTitle": return "title"; break;
		case "itemLink": return this.type == "rss" ? "link" : "link/attribute::href"; break;
		case "itemDescription": return this.type == "rss" ? "description" : "content"; break;
		case "itemGuid": return this.type == "rss" ? "guid" : "id"; break;
		case "itemPubDate": return this.type == "rss" ? "pubDate" : "updated"; break;
	}
}

// based on dojo toolkit's dojo.date.stamp.fromISOString() function
Feed.dateFromRFC3339 = function(str) {
	if (!Feed.dateFromRFC3339.rfcRE) {
		Feed.dateFromRFC3339.rfcRE = /^(\d{4})-(\d{2})-(\d{2})T(\d{2}):(\d{2})(?::(\d{2})(.\d+)?)?((?:[+-](\d{2}):(\d{2}))|Z)?$/;
	}

	var match = Feed.dateFromRFC3339.rfcRE.exec(str);
	var result = null;

	if (match) {
		match.shift();
		match[1]--; // Javascript Date months are 0-based
		match[6] && (match[6] *= 1000); // Javascript Date expects fractional seconds as milliseconds
		
		result = new Date(match[0], match[1], match[2], match[3], match[4], match[5]?match[5]:0, match[6]?match[6]:0);

		var offset = 0;
		var zoneSign = match[7] && match[7].charAt(0);
		if (zoneSign != 'Z') {
			offset = ((match[8] || 0) * 60) + (Number(match[9]) || 0);
			if (zoneSign != '-') { offset *= -1; }
		}
		if (zoneSign) {
			offset -= result.getTimezoneOffset();
		}
		if (offset) {
			result.setTime(result.getTime() + offset * 60000);
		}
	}
	
	return result; // Date or null
}


// AggFeed - special case of the Feed object
function AggFeed() {
	this.base = Feed;
	this.base(); // call the Feed constructor
	
	this.title = loc('allFeeds');
	this.description = loc('allFeedsDescription');
	this.url = 'allFeeds';
	
	this.refreshTimer.ticking = false;
	delete this.refreshTimer;
	delete this.request;
	
}
AggFeed.prototype = new Feed;

// overrides

AggFeed.prototype.type = "allFeeds";

// Load AggFeed with values from the other feeds
AggFeed.prototype.refresh = function() {
	this.items = new Array();
	for (var i = 0; i < feedsPane.rows.length; i++) {
		var feed = feedsPane.rows[i].feed;
		if (feed && feed != this && feed.items) {
			for (var j = 0; j < feed.items.length; j++) {
				this.items.push(feed.items[j]);
			}
		}
	}
	
	this.items.sort(Feed.sortFunc);
	this.updateReadItems();
}


AggFeed.prototype.updateDisplay = function() {
	if (this.row) {
		Feed.prototype.updateDisplay.call(this);
	}
}

AggFeed.prototype.updateReadItems = function() {
	printFunction('updateReadItems');
	this.readItems = 0;
	for (var i = 0; i < this.items.length; i++) {
		if (this.items[i].read) {
			this.readItems++;
		}
	}
}




allFeeds = new AggFeed();



// sorts feeds in reverse chronological order (newest at top of list)
Feed.sortFunc = function(a, b) {
	return b.pubDate - a.pubDate;
}