﻿/*
	NewsStand v. 1.25
	Quickly access information from your favorite RSS sources.

	Copyright © 2008 WidgetWorld.  All Rights Reserved.
*/

var feedsLoaded, widgetInitialized, widgetLuminance, separatorHoffset, showReadItems, tRow, safeModeUrl;
var readItems = new Array();
var adFeedRequest = new XMLHttpRequest(); // Ad request object - used to fetch data from the "ad feed"
var sheetMode = false;

const MIN_BACKGROUND_WIDTH = 414;
const MAX_BACKGROUND_WIDTH = 900;
const MIN_BACKGROUND_HEIGHT = 265;
const MAX_BACKGROUND_HEIGHT = 685;

const MIN_SEPARATOR_OFFSET = 150;
const MAX_SEPARATOR_OFFSET = 250;

const AD_FEED_URL = "";

const DEFAULT_FEEDS = [
	"http://www.elertgadget.com/igxml.php?gid=2009"
];

// Version Cheking Pieces
const VERSION_NUMBER = widget.version;
const VERSION_SHORTNAME = "elertgadget";
const VERSION_URL = "http://elertgadget.com";
const VERSION_WIDGETNAME = "elertgadget";

// Include additional files
include("Debug.js");
include("VersionChecker.js");
include("ResizableImage.js");
include("Tooltip.js");
include("Tray.js");
include("Pane.js");
include("ListRow.js");
include("HtmlEntitiesConvert.js");
include("FeedSettings.js");
include("Feed.js");
include("FeedItem.js");
include("Controls.js");
include("Alerts.js");
include("Dock.js");

// Localize preferences
preferences.showReadItems.title = loc("showReadItemsPref");
preferences.showReadItems.description = loc("showReadItemsPrefDescription");
preferences.confirmMarkAll.title = loc("confirmMarkAllPref");
preferences.confirmMarkAll.description = loc("confirmMarkAllPrefDescription");
preferences.autoCheckVersion.title = loc("autoCheckVersionPref");
preferences.autoCheckVersion.description = loc("autoCheckVersionPrefDescription");
preferenceGroups.appearance.title = loc("appearancePrefGroup");
preferences.font.title = loc("fontPref");
preferences.skin.title = loc("skinPref");
preferences.skin.option = [loc("basicColorizationSkin"), loc("fullColorizationSkin")];
preferences.skin.optionValue = ["1bc", "2fc"];
preferences.color.title = loc("colorPref");
preferences.descriptionsType.title = loc("descriptionsTypePref");
preferences.descriptionsType.option = [loc("standardTooltipLabel"), loc("customTooltipLabel"), loc("pullOutTrayLabel")];
preferences.descriptionsType.optionValue = ["0", "1", "2"];

/************************************************************************
	Main Functions
*************************************************************************/

function Initialize()
{
	printFunction("Initialize");

	SetDockLabel("");

	// Black text shadow
	blackShadow = new Shadow();
	blackShadow.color = "#000000";
	blackShadow.vOffset = -1;
	blackShadow.opacity = 128;

	// White text shadow
	whiteShadow = new Shadow();
	whiteShadow.color = "#FFFFFF";
	whiteShadow.vOffset = 1;
	whiteShadow.opacity = 128;
	
	// Main Frame
	mainFrame = new Frame(mainWindow);
	mainFrame.hOffset = 123;
	mainFrame.vOffset = 80;

	// Widget's background
	background = new ResizableImage("background", "Resources/Images/Background", true, mainFrame);
	background.hOffset = 0;
	background.vOffset = 0;
	background.onDragDrop = onDragDrop;

	// The frame on which the feeds pane is placed
	feedsFrame = new Frame();
	feedsFrame.hOffset = 21;
	feedsFrame.vOffset = 20;
	feedsFrame.animation = new CustomAnimation(1, UpdateFeedStateAnimation);
	mainFrame.appendChild(feedsFrame);

	// Feeds pane - where feeds are listed
	feedsPane = new Pane("feedsPane", 1, feedsFrame);
	feedsPane.reorderable = true;
	feedsPane.hOffset = 0;
	feedsPane.vOffset = 0;

	// Feeds counter - the number of available feeds
	feedsCounter = new Text();
	feedsCounter.hOffset = 10;
	feedsCounter.vOffset = 17;
	feedsCounter.color = "#FFFFFF";
	feedsCounter.shadow = blackShadow;
	feedsCounter.size = 12;
	feedsCounter.truncation = "end";
	feedsFrame.appendChild(feedsCounter);

	// Add a feed button
	addButton = new MiniButton("addButton", 5, 0, "Resources/Images/addButton", feedsFrame, "AddFeed();");

	// Delete selected feed button
	deleteButton = new MiniButton("deleteButton", 25, 0, "Resources/Images/deleteButton", feedsFrame, "DeleteFeed(feedsPane.selectedRow.feed, true);");

	// Edit selected feed  button
	editButton = new MiniButton("editButton", 45, 0, "Resources/Images/editButton", feedsFrame, "EditFeed(feedsPane.selectedRow.feed);");

	// The separator "gripper" image
	separator = new Image();
	separator.src = "Resources/Images/resizer.png";
	separator.hAlign = "center";
	mainFrame.appendChild(separator);

	// Load separator position from memory
	separatorHoffset = Math.min(MAX_SEPARATOR_OFFSET, Math.max(MIN_SEPARATOR_OFFSET, preferences.separatorPosition.value));

	// Separator strip - used to allow resizing  of the panes horizontally by dragging the separator between them
	transStrip = new Image();
	transStrip.src = "Resources/Images/transStrip.png";
	transStrip.hOffset = separatorHoffset;
	transStrip.vOffset = feedsFrame.vOffset;
	transStrip.tracking = "rectangle";
	transStrip.onMouseDown = "mouseX = system.event.x;";
	transStrip.onMouseDrag = "MoveSeparator(transStrip.hOffset + system.event.x - mouseX);";
	mainFrame.appendChild(transStrip);

	// Content pane - where articles and the feed's additional information are shown
	contentPane = new Pane("contentPane", 0, mainFrame);
	contentPane.hOffset = transStrip.hOffset + transStrip.width;
	contentPane.vOffset = 20;

	throbber = new Throbber(mainFrame);
	throbber.opacity = 0;
	
	// Feed title - the title of the selected feed
	feedTitle = new Text();
	feedTitle.vOffset = 37;
	feedTitle.color = "#FFFFFF";
	feedTitle.shadow = blackShadow;
	feedTitle.size = 11;
	feedTitle.truncation = "end";
	mainFrame.appendChild(feedTitle);

	// Previous feed button
	previousButton = new MiniButton("previousButton", 0, 26, "Resources/Images/previousButton", mainFrame, "PreviousFeed();");

	// Next feed button
	nextButton = new MiniButton("nextButton", 0, 26, "Resources/Images/nextButton", mainFrame, "NextFeed();");

	// Show or hide feeds pane button
	paneButton = new MiniButton("paneButton", 0, 0, "Resources/Images/showPaneButton", mainFrame, "SetFeedsPaneState(!separator.opacity, true);");

	// Refresh current feed button
	refreshButton = new MiniButton("refreshButton", 0, 0, "Resources/Images/refreshButton", mainFrame, "onRefreshButtonClick();");

	// Mark current feed's items as read button
	markAllButton = new MiniButton("markAllButton", 0, 0, "Resources/Images/markAllButton", mainFrame, "onMarkAllButtonClick();");
	
	// Feed updated text - shows when the selected feed was last updated
	feedUpdated = new Text();
	feedUpdated.hAlign = "right";
	feedUpdated.color = "#FFFFFF";
	feedUpdated.shadow = blackShadow;
	feedUpdated.size = 10;
	feedUpdated.truncation = "end";
	mainFrame.appendChild(feedUpdated);

	// Ad text
	ad = new Text();
	ad.hAlign = "center"
	ad.size = 11;
	ad.truncation = "end";
	ad.onClick = "if(ad.link) openURL(ad.link);";
	ad.fadeIn = new FadeAnimation(ad, 255, 800, animator.kEaseOut);
	mainFrame.appendChild(ad);

	// Resize gripper - used to resize the Widget
	resize = new Image();
	resize.src = "Resources/Images/resize.png";
	resize.tracking = "rectangle";
	resize.onMouseDown = "mouseX = system.event.x; mouseY = system.event.y;";
	resize.onMouseDrag = "SetSize(mainFrame.width + system.event.x - mouseX, mainFrame.height + system.event.y - mouseY);";
	mainFrame.appendChild(resize);

	// The overlay image is used to disable the Widget's main UI while "sheets" are being shown
	overlay = new ResizableImage("overlay", "Resources/Images/Overlay", true, mainFrame);
	overlay.hOffset = 0;
	overlay.vOffset = 0;
	overlay.opacity = 0;
	overlay.fade = new CustomAnimation(1, UpdateOverlayOpacity);
	overlay.fade.duration = 800;
	overlay.onClick = "print();";
	overlay.onContextMenu = function(){mainWindow.contextMenuItems = null;}

	// The "sheet frame" is where "sheets" are placed
	sheetFrame = new Frame();
	sheetFrame.hAlign = "center";
	sheetFrame.vOffset = 10;
	mainFrame.appendChild(sheetFrame);

	CreateTooltip();
	if(preferences.descriptionsType.value == "2") CreateTray();
	
	// Set the window's and panes' sizes
	SetSize(preferences.widgetWidth.value, preferences.widgetHeight.value);
	SetFeedsPaneState(parseBool(preferences.showFeedsPane.value), false);

	// Load read items list from memory
	readItems = preferences.readItems.value.split("\n");

	// Set "feeds pane"'s "events"
	feedsPane.onRowRemoved = function(row){
		SetDockLabel("");
		if (row != allFeeds.row) { 
			RecountFeeds(true); 
		}
		if(row == feedsPane.selectedRow){
			contentPane.setListState(false, true, true);
		}
	}
	feedsPane.onRowClicked = function(row)
	{
		if(preferences.descriptionsType.value == "2")
		{
			if(!trayFrame.open || tRow != row)
			{
				OpenTray(row, row.feed.getTitle(true), row.feed.getDescription(), !row.feed.refreshing && row.feed.feedUpdated ? timeToString(row.feed.feedUpdated.getTime()) : "");
			}
			else CloseTray(true);
		}
	}
	feedsPane.onRowSelected = function(row){ShowFeed(row ? row.feed : null);}
	feedsPane.onRowContextMenu = function(row){row.setContextMenu(GetFeedMenu(row));}
	feedsPane.onDragDrop = onDragDrop;
	feedsPane.onMouseEnterRow = function(row)
	{
		if(!row.feed) return;

		if(preferences.descriptionsType.value == "1")
		{
			ShowTooltip(row, row.feed.getTitle(true), row.feed.getDescription(), !row.feed.refreshing && row.feed.feedUpdated ? timeToString(row.feed.feedUpdated.getTime()) : "");
		}
	}
	feedsPane.onMouseMoveOverRow = function(row){MoveTooltip(system.event.hOffset, system.event.vOffset);}
	feedsPane.onScrollbarValueChanged = HideTooltip;
	feedsPane.onMouseEnterScrollbar = HideTooltip;
	feedsPane.onMouseExitRow = HideTooltip;
	feedsPane.onMouseWheel = HideTooltip;

	// Set "content pane"'s "events"
	contentPane.onRowRemoved = function(row){UpdateFeedTitle();}
	contentPane.onRowSelected = function(row)
	{
		if(preferences.descriptionsType.value == "2")
		{
			if(trayFrame.open && row.item && tRow == row)
			{
				OpenItem(row);
				CloseTray(true);
			}
			else OpenTray(row, row.title.data, row.item.description, timeToString(row.item.pubDate));
		}
		else if(row.item) OpenItem(row);
	}
	contentPane.onRowContextMenu = function(row){if(row.item) row.setContextMenu(GetItemMenu(row));}
	contentPane.onDragDrop = onDragDrop;
	contentPane.onMouseEnterRow = function(row)
	{
		if(!row.item) return;
		
		if(preferences.descriptionsType.value == "1")
		{
			ShowTooltip(row, row.title.data, row.item.description, timeToString(row.item.pubDate));
		}
	}
	contentPane.onMouseMoveOverRow = function(row){MoveTooltip(system.event.hOffset, system.event.vOffset);}
	contentPane.onMouseWheel = HideTooltip;
	contentPane.onMouseEnterScrollbar = HideTooltip;
	contentPane.onMouseExitRow = HideTooltip;
	
	ApplyPreferences();
	LoadFeeds(true);
	RecountFeeds();

	// Set last selected feed:
	var selectedFeed = preferences.selectedFeed.value; // Load last selected feed from memory
	if(selectedFeed) // Is there a feed saved in memory?
	{
		if (selectedFeed == 'allFeeds') {
			feedsPane.selectRow(allFeeds.row);
			ShowFeed(allFeeds);
		} else {
			for(var ind in feedsPane.rows) // Iterate through all rows to find if the feed saved in memory appears in the list
			{
				try
				{
					if(feedsPane.rows[ind].feed.url == selectedFeed)
					{
						feedsPane.selectRow(ind, true);
						break;
					}
				}
				catch(e)
				{
					printError("Initialize", "Couldn't obtain feed link from row: #" + ind + ".", e);
				}
			}
		}
	}
	if(!feedsPane.selectedRow) feedsPane.rows.length > 0 ? feedsPane.selectRow(0, true) : ShowFeed(); // If no row is selected, select the first one or none, if no rows are available}

	RefreshFeeds();
	RecountFeeds();
	RefreshAd();

	if(parseBool(preferences.autoCheckVersion.value)) CheckVersion(false);  // Check for version updates if needed

	widgetInitialized = true; // The Wiget has been initialized
}

function Unload()
{
	printFunction("Unload");

	// Stop refreshing feeds
	for(var ind in feedsPane.rows)
	{
		if(feedsPane.rows[ind].feed.request) feedsPane.rows[ind].feed.request.abort();
	}

	// Stop "ad feed" if it is currently being refreshed
	adFeedRequest.abort();

	// Save preferences
	preferences.widgetWidth.value = mainFrame.width;
	preferences.widgetHeight.value = mainFrame.height;
	preferences.separatorPosition.value = separatorHoffset;
	preferences.selectedFeed.value = feedsPane.selectedRow ? feedsPane.selectedRow.feed.url : "";
	preferences.readItems.value = readItems.join("\n");

	SaveFeeds();
}

function ApplyPreferences()
{
	printFunction("ApplyPreferences");
	
	if(preferences.descriptionsType.value == "2") CreateTray();
	else RemoveTray();
	
	SetTooltipsEnabled(!overlay.opacity, !overlay.opacity && preferences.descriptionsType.value == "0");

	var color = getImageColor();

	// Calculate Widegt luminance
	var r = parseInt(color.substr(1, 2), 16);
	var g = parseInt(color.substr(3, 2), 16);
	var b = parseInt(color.substr(5, 2), 16);

	widgetLuminance = Math.round((r * 0.3) + (g * 0.59) + (b * 0.11)) > 128;
	
	// Colorize outer images
	background.colorize =
	resize.colorize =
	separator.colorize = getImageColor();

	// Colorize inner images
	previousButton.colorize =
	nextButton.colorize =
	refreshButton.colorize =
	markAllButton.colorize =
	addButton.colorize =
	deleteButton.colorize =
	editButton.colorize =
	paneButton.colorize = getImageColor(true);

	// Update "ad" appearance
	ad.color = getTextColor();
	ad.shadow = getShadowColor();
	ad.font = feedTitle.font = feedsCounter.font = feedUpdated.font = preferences.font.value;

	// Update panes appearance
	feedsPane.updateAppearance();
	contentPane.updateAppearance();

	// Update additional frames' appearance
	if(fsFrame && sheetFrame.sheet == fsFrame) UpdateFSWindowAppearance();
	if(alertFrame && sheetFrame.sheet == alertFrame) UpdateAlertAppearance();
	
	if(showReadItems != null && showReadItems != parseBool(preferences.showReadItems.value))
	{
		var feed = feedsPane.selectedRow.feed;
		
		for(var ind in feed.items)
		{
			if(feed.items[ind].read)
			{
				if(showReadItems) RemoveItem(feed.items[ind]);
				else AddItem(feed.items[ind], feed.items[ind].id);
			}
		}
	}
	
	UpdateTrayAppearance();
	
	showReadItems = parseBool(preferences.showReadItems.value);
	
	// Start or stop the timer which checks for version updates
	versionUpdateTimer.ticking = parseBool(preferences.autoCheckVersion.value);
}

// Listen to a "Tell Widget" command
function Listen()
{
	printFunction("Listen");

	// Check if a "Load Feed" command is called
	var feed = system.event.data.toLowerCase().match(/^load_feed ([^>]+)$/);
	if(feed)
	{
		// Check if the feed is already in the list
		for(var ind in feedsPane.rows)
		{
			try
			{
				if(feedsPane.rows[ind].feed.url == feed[1])
				{
					if(feedsPane.rows[ind] != feedsPane.selectedRow) feedsPane.selectRow(ind, true);
					return;
				}
			}
			catch(e)
			{
				printError("Listen", "Couldn't obtain feed link from row: #" + ind + ".", e);
			}
		}

		AddFeed(feed[1]);
	}
}

// If the user drops a URL, add a new feed based on it
function onDragDrop()
{
	if(sheetMode) return;
	
	if(!overlay.opacity && system.event.data[0] == "urls" || system.event.data[0] == "string")
	{
		var url = system.event.data[1].match(/(http:|https:|feed:)\/\/(\S+)/)
		if(url) AddFeed(url[2]);
	}
}

function onRefreshButtonClick() {
	if (feedsPane.selectedRow.feed == allFeeds) {
		RefreshFeeds();
	} else {
		feedsPane.selectedRow.feed.refresh();
	}
}

function onMarkAllButtonClick() {
	MarkFeedItemsAsRead(feedsPane.selectedRow.feed, true, true);
}

// Set Widget's size
function SetSize(width, height)
{
	mainFrame.width = background.width = Math.min(MAX_BACKGROUND_WIDTH, Math.max(MIN_BACKGROUND_WIDTH, width));
	mainFrame.height = background.height = Math.min(MAX_BACKGROUND_HEIGHT, Math.max(MIN_BACKGROUND_HEIGHT, height));

	feedsFrame.height = feedsPane.height = mainFrame.height - 54;

	contentPane.width = mainFrame.width - 20 - contentPane.hOffset;
	contentPane.height = feedsFrame.height;

	separator.vOffset = feedsFrame.vOffset + feedsFrame.height/2;
	transStrip.height = feedsFrame.height;

	throbber.hOffset = contentPane.hOffset + contentPane.width/2;
	throbber.vOffset = contentPane.vOffset + contentPane.height/2;
	throbber.size = Math.min(110, contentPane.width - 10);
	
	editButton.vOffset = deleteButton.vOffset = addButton.vOffset = feedsFrame.height - 20;
	markAllButton.vOffset = refreshButton.vOffset = paneButton.vOffset = mainFrame.height - 54;
	
	nextButton.hOffset = contentPane.hOffset + contentPane.width - nextButton.width - 5;
	previousButton.hOffset = nextButton.hOffset - 20;

	resize.hOffset = mainFrame.width - 30;
	resize.vOffset = mainFrame.height - 30;

	feedTitle.width = previousButton.hOffset - contentPane.hOffset - 11;

	feedUpdated.hOffset = contentPane.hOffset + contentPane.width - 5;
	feedUpdated.vOffset = mainFrame.height - 45;

	feedUpdated.width = mainFrame.width - markAllButton.hOffset - markAllButton.width - 28;

	ad.hOffset = mainFrame.width/2;
	ad.vOffset = mainFrame.height - 19;
	
	if(trayFrame && trayFrame.open) ResizeTray(null, height - 40);
	RepositionTray();
}

// Move the separator between the panes
function MoveSeparator(hOffset, force)
{
	transStrip.hOffset = force ? hOffset : Math.min(MAX_SEPARATOR_OFFSET, Math.max(MIN_SEPARATOR_OFFSET, hOffset));
	if(!force) separatorHoffset = transStrip.hOffset;

	feedsFrame.width = feedsPane.width = transStrip.hOffset - feedsFrame.hOffset;
	feedsCounter.width = feedsFrame.width - 20;
	separator.hOffset = feedsFrame.hOffset + feedsFrame.width + 5;

	contentPane.hOffset = transStrip.hOffset + transStrip.width;
	contentPane.width = mainFrame.width - 20 - contentPane.hOffset;

	throbber.hOffset = contentPane.hOffset + contentPane.width/2;
	throbber.vOffset = contentPane.vOffset + contentPane.height/2;
	throbber.size = Math.min(110, contentPane.width - 10);
	
	paneButton.hOffset = contentPane.hOffset + 5;
	refreshButton.hOffset = paneButton.hOffset + 20;
	markAllButton.hOffset = refreshButton.hOffset + 20;

	feedTitle.hOffset = contentPane.hOffset + 10;
	feedTitle.width = previousButton.hOffset - contentPane.hOffset - 11;

	feedUpdated.width = mainFrame.width - markAllButton.hOffset - markAllButton.width - 28;
}

// Update feeds display
function RecountFeeds(refreshAggregate)
{
	printFunction("RecountFeeds");
	
	
	var feedCount = feedsPane.rows.length;
	var hideAllFeeds = false;
	
	if (allFeeds.row) {
		feedCount--;
	}
	// print('feedCount: ' + feedCount);
	
	if (feedCount > 0) {
		if (feedCount == 1) {
			feedsCounter.data = loc("1feed");
			hideAllFeeds = true;
		} else {
			feedsCounter.data = loc("numFeeds").replace("%num%", feedCount);
			
			if (refreshAggregate && !allFeeds.row) {
				AddFeed({url:'allFeeds'});
			}
		}
	} else {
		feedsCounter.data = loc("noFeeds");
		hideAllFeeds = true;
	}
	
	if (refreshAggregate && hideAllFeeds && allFeeds.row) {
		var rowToRemove = allFeeds.row;
		allFeeds.row = null;
		feedsPane.removeRow(rowToRemove);
	}
	
	if (feedsPane.selectedRow != allFeeds.row) {
		deleteButton.setEnabled(feedsPane.rows.length > 0);
		editButton.setEnabled(feedsPane.rows.length > 0);
	}
}

// Update the feed's title label and the "current feed buttons"
function UpdateFeedTitle()
{
	printFunction("UpdateFeedTitle");
	
	var feed = feedsPane.selectedRow ? feedsPane.selectedRow.feed : null;

	if (feed) {
	
		if (feed == allFeeds) {
			allFeeds.updateReadItems();
		}
	
		feedsPane.selectedRow.setActive(feed.readItems < feed.items.length, widgetInitialized);
		
		if (feed.error) {
			feedTitle.data = feed.getTitle(true);
		} else if(feed.refreshing) {
			feedTitle.data = loc("loading");
			if(!throbber.on) throbber.start(widgetInitialized);
		} else {
			feedTitle.data = feed.getTitle();
			if(feed.items.length > 0)
			{
				feedTitle.data += " (" + loc("itemsCount").replace("%total%", feed.items.length).replace("%unreadNum%", feed.items.length - feed.readItems) + ")";
			}
			else feedTitle.data += " (" + loc("noItems") + ")";
		}
	} else {
		feedTitle.data = "";
	}
	
	if (feed != allFeeds) {
		allFeeds.updateReadItems();
		allFeeds.updateDisplay();
	}
	
	previousButton.setEnabled(feedsPane.selectedRow && feedsPane.rows.length > 1);
	nextButton.setEnabled(feedsPane.selectedRow && feedsPane.rows.length > 1);
	refreshButton.setEnabled(feed && !feed.refreshing);
	markAllButton.setEnabled(feed && !feed.error && !feed.refreshing && feed.items.length > 0 && feed.readItems < feed.items.length);
	
	UpdateDock();
}

// Update the "Updated..." text
function UpdateUpdatedText()
{
	printFunction("UpdateUpdatedText");

	var row = feedsPane.selectedRow;

	if(row && !row.feed.refreshing && !row.feed.error && row.feed.feedUpdated instanceof Date)
	{
		updatedTextTimer.ticking = true;
		feedUpdated.data = loc("updated") + " " + timeToString(row.feed.feedUpdated.getTime());
	}
	else
	{
		feedUpdated.data = "";
		updatedTextTimer.ticking = false;
	}
	
	if(row &&  tRow == row)
	{
		if(preferences.descriptionsType.value == "1")
		{
			UpdateTooltip(row.feed.getTitle(true), row.feed.getDescription(), !row.feed.refreshing && row.feed.feedUpdated ? timeToString(row.feed.feedUpdated.getTime()) : "");
		}
		
		if(preferences.descriptionsType.value == "2")
		{
			UpdateTray(row.feed.getTitle(true), row.feed.getDescription(), !row.feed.refreshing && row.feed.feedUpdated ? timeToString(row.feed.feedUpdated.getTime()) : "");
		}
	}
}

// Show or hide the feeds pane
function SetFeedsPaneState(show, animate)
{
	printFunction("SetFeedsPaneState");

	if(animate)
	{
		// Prepare animation
		var animation = feedsFrame.animation;
		animation.duration = 600;
		if(show && !separator.opacity)
		{
			animation.startSeparatorPosition = feedsFrame.hOffset - transStrip.width;
			animation.endSeparatorPosition = separatorHoffset;
			animation.show = true;
		}
		else if(separator.opacity)
		{
			animation.startSeparatorPosition = separatorHoffset;
			animation.endSeparatorPosition = feedsFrame.hOffset - transStrip.width;
			animation.show = false;
		}

		// Play animation if needed
		if(animation) animator.runUntilDone(animation);
	}
	else
	{
		separator.opacity = transStrip.opacity = show ? 255 : 0;
		MoveSeparator(show ? separatorHoffset : feedsFrame.hOffset - transStrip.width, true);

		paneButton.src = show ? "Resources/Images/hidePaneButton" : "Resources/Images/showPaneButton";
		paneButton.tooltip = show ? loc("hideFeedsTP") : loc("showFeedsTP");

		preferences.showFeedsPane.value = show;
	}
}

// The "Set Feeds Pane State" animation
function UpdateFeedStateAnimation()
{
	var	now = animator.milliseconds;
	var	t = Math.max(0, now - this.startTime);
	var percent = t/this.duration;

	MoveSeparator(animator.ease(this.startSeparatorPosition, this.endSeparatorPosition, percent, animator.kEaseOut), true);

	if(animator.milliseconds >= this.startTime + this.duration)
	{
		SetFeedsPaneState(this.show, false);
		return false;
	}

	return true;
}

// Enable or disable tooltips for main objects and list rows
function SetTooltipsEnabled(main, rows)
{
	printFunction("SetTooltipsEnabled");
	
	// Main objects
	addButton.tooltip = main ? loc("addFeedTP") : "";
	deleteButton.tooltip = main ? loc("deleteFeedTP") : "";
	editButton.tooltip = main ? loc("editFeedTP") : "";
	paneButton.tooltip = main ? (parseBool(preferences.showFeedsPane.value) ? loc("hideFeedsTP") : loc("showFeedsTP")) : "";
	previousButton.tooltip = main ? loc("showPreviousFeed") : "";
	nextButton.tooltip = main ? loc("showNextFeed") : "";
	refreshButton.tooltip = main ? ((feedsPane.selectedRow && feedsPane.selectedRow == allFeeds.row) ? loc("refreshAllFeeds") : loc("refreshCurrentFeed")) : "";
	markAllButton.tooltip = main ? ((feedsPane.selectedRow && feedsPane.selectedRow == allFeeds.row) ? loc("markAllFeedItemsRead") : loc("markFeedItemsRead")) : "";
		
	// Rows
	for(var ind in feedsPane.rows)
	{
		feedsPane.rows[ind].setTooltip(rows ? feedsPane.rows[ind].feed.getDescription() : "");
	}
	for(var ind in contentPane.rows)
	{
		contentPane.rows[ind].setTooltip((rows && contentPane.rows[ind].item) ? getShortString(contentPane.rows[ind].item.description, 190) : "");
	}
}

/************************************************************************
	Feeds and Items Management
*************************************************************************/

// Show a feed - if no feed is passed, the Widget switches to "No Feeds Available" mode.
function ShowFeed(feed)
{
	printFunction("ShowFeed");
	if(contentPane.rows.length > 0) contentPane.clearList();
	
	if (feed == allFeeds) {
		allFeeds.refresh();
		deleteButton.setEnabled(false);
		editButton.setEnabled(false);
	} else {
		deleteButton.setEnabled(true);
		editButton.setEnabled(true);
	}
	
	if (feed && !feed.error && !feed.refreshing) {
		if(feed.items.length > 0)
		{
			// Load feed items
			for(var ind in feed.items)
			{
				if(parseBool(preferences.showReadItems.value) || !feed.items[ind].read)
				{
					AddItem(feed.items[ind]);
				}
			}

			contentPane.list.home();
		}
	} else if (feed && feed.error) {
		contentPane.addRow(loc("couldNotLoadFeedInfo"));
	}
	
	UpdateFeedTitle();
	UpdateUpdatedText();
	
	SetTooltipsEnabled(true, preferences.descriptionsType.value == "0");
	
	// Fade in items list
	contentPane.setListState(true, widgetInitialized);
	if(!feed || !feed.refreshing) throbber.stop(widgetInitialized);

}

// Add a new feed: "object" can be a null value to load an empty "Feed Settings" window, a feed URL string to load the "Feed Settings" window using a specified feed URL
// or an "RSS Settings" object to add a new feed based on specified settings.
function AddFeed(object, simpleRecount, ignoreDoubles)
{
	printFunction("AddFeed");

	if (object == null || typeof(object) == "string") {
		ShowFeedSettings(object);
	} else {
		if (object.url == 'allFeeds') {
			// special allFeeds row
			if (allFeeds.row) {
				// if row already exists, reorder it
				feedsPane.reorderRow(allFeeds.row, feedsPane.rows.length);
			} else {
				var id = feedsPane.rows.length;
				var allFeedsRow = new ListRow(id, loc('allFeeds'), feedsPane);
				// allFeedsRow.reorderable = false;
				allFeedsRow.feed = allFeeds;
				feedsPane.rows.splice(id, 0, allFeedsRow);
				allFeeds.row = allFeedsRow;
			}
		} else {
			// normal feed row
			
			// Check if the feed is already in the list
			for(var ind in feedsPane.rows)
			{
				try
				{
					if(feedsPane.rows[ind].feed.url == object.url)
					{
						if(ignoreDoubles) return;
						else
						{
							// The feed is already in the list - the user can choose to overwrite it using the settings sent from the "Feed Settings" window
							Alert(loc("feedAlreadyInListMsg"), loc("overwrite"), loc("cancel"), function(){EditFeed(feedsPane.rows[ind].feed, object);});
							return;
						}
					}
				}
				catch(e)
				{
					printError("AddFeed", "Couldn't obtain feed link from row: #" + ind + ".", e);
				}
			}
			
			// Add feed based on the passed "settings" object:
			var feed = new Feed(object); // Create a new feed object
			var row = feedsPane.addRow(); // Add a new row

			row.feed = feed;
			feed.row = row;
		}
		RecountFeeds(!simpleRecount);

		// Refresh feed if the Widget has finished initializing
		if(widgetInitialized && feed) feed.refresh();

		SaveFeeds();
	}
}



function DeleteFeed(feed, alert)
{
	printFunction("DeleteFeed");

	if(alert) Alert(loc('deleteFeedMsg').replace('%feedName%', getShortString(feed.getTitle(), 50)), loc('delete'), loc('cancel'), function(){DeleteFeed(feed);});
	else
	{
		feed.refreshTimer.ticking = false;
		feed.request.abort();
		feedsPane.removeRow(feed.row.id, true);
		RecountFeeds();
		SaveFeeds();
	}
}

function EditFeed(feed, settings)
{
	printFunction("EditFeed");

	if(!settings) ShowFeedSettings(feed); // If no settings are passed, get them from "Feed Settings" window
	else
	{
		// Apply settings to feed
		for(var ind in settings)
		{
			feed[ind] = settings[ind];
		}

		feed.refresh();
	}
}

// Refresh all feeds
function RefreshFeeds()
{
	printFunction("RefreshFeeds");

	for(var ind in feedsPane.rows)
	{
		if (feedsPane.rows[ind].feed.refresh) {
			feedsPane.rows[ind].feed.refresh();
		}
	}
}

// Mark all feed items as read/unread
function MarkFeedItemsAsRead(feed, read, sheetMode)
{
	printFunction("MarkFeedItemsAsRead");

	if(parseBool(preferences.confirmMarkAll.value) && sheetMode)
	{
		Alert(((feed == allFeeds) ? loc('markAllFeedItemsMsg') : loc('markFeedItemsMsg')).replace('%feedName%', getShortString(feed.getTitle(), 70)).replace("%status%", read ? loc("read") : loc("unread")), loc('yes'), loc('no'), function(){MarkFeedItemsAsRead(feed, read);});
	}
	else
	{
		contentPane.list.scrollY = 0;
		
		for(var ind in feed.items)
		{
			feed.items[ind].setRead(read, parseBool(preferences.showReadItems.value));
		}
		
		if (feed == feedsPane.selectedRow.feed) {
			UpdateFeedTitle();
		}
		allFeeds.refresh();
		
		allFeeds.updateDisplay();
	}
}

// Switch to next feed
function NextFeed()
{
	printFunction("NextFeed");

	if(feedsPane.rows.length > 1)
	{
		var id = feedsPane.selectedRow ? (feedsPane.selectedRow.id + 1) : 0;
		if(id == feedsPane.rows.length) id = 0;
		feedsPane.selectRow(id, true);
	}
}

// Switch to previous feed
function PreviousFeed()
{
	printFunction("PreviousFeed");

	if(feedsPane.rows.length > 1)
	{
		var id = feedsPane.selectedRow ? feedsPane.selectedRow.id - 1 : 0;
		if(id == -1) id = feedsPane.rows.length - 1;
		feedsPane.selectRow(id, true);
	}
}

// Add a row to the content pane
function AddItem(feedItem, id, animate)
{
	var row = contentPane.addRow(feedItem.title, id, animate);
	row.item = feedItem;
	feedItem.row = row;
	if(feedItem.read) row.setActive(false);
	if(!overlay.opacity && preferences.descriptionsType.value == "0") row.setTooltip(getShortString(feedItem.description, 190));
}

// Remove a row from the content pane
function RemoveItem(feedItem, animate)
{
	contentPane.removeRow(feedItem.row.id, animate);
	feedItem.row = null;
}

// Open an item's link in browser
function OpenItem(row)
{
	printFunction("OpenItem");

	// Mark item as read if needed
	if(!row.item.read)
	{
		row.item.setRead(true, true);
		UpdateFeedTitle();
	}

	openURL(row.item.link);
}

// Load feeds from memory
function LoadFeeds(checkCrashLog, openInSafeMode)
{
	printFunction("LoadFeeds");

	if(checkCrashLog && filesystem.itemExists(system.widgetDataFolder + "/log.txt"))
	{
		// Check if a feed's URL appears in the crash log. If it is, ask the user whether to load it in safe mode or not.
		var url = filesystem.readFile(system.widgetDataFolder + "/log.txt");
		if(url && url.length > 0)
		{
			safeModeUrl = url;
			Alert(loc("openInSafeModeMsg"), loc("yes"), loc("no"), function(){LoadFeeds(false, true);}, function(){LoadFeeds(false, false);});
			return;
		}
	}
	
	if(preferences.feeds.value) // Make sure there's data saved in memory
	{
		var doc = XMLDOM.parse(preferences.feeds.value);

		if(doc != null)
		{
			try
			{
				var feedNodes = doc.evaluate("Feeds/Feed");

				var feedNodesNumber = feedNodes.length;

				if(feedNodesNumber && feedNodesNumber > 0) // Make sure there are feeds to load
				{
					for(var id = 0; id < feedNodesNumber; id++)
					{
						try
						{
							var feedAttributes = feedNodes.item(id).attributes;

							var settings = new Object();

							for(var i = 0; i < feedAttributes.length; i++)
							{
								settings[feedAttributes.item(i).name] = feedAttributes.item(i).value;
							}
							
							// If the user wants to load invalid feeds in safe mode and the current feed
							// was marked as invalid, load it in safe mode.
							if(openInSafeMode && safeModeUrl && safeModeUrl == settings.url)
							{
									settings.safeMode = true;
									safeModeUrl = null;
							}							

							AddFeed(settings, true);
						}
						catch(e)
						{
							printError("LoadFeeds", "Couldn't load feed #" + id + ".", e);
						}
					}
				}
			}
			catch(e)
			{
				printError("LoadFeeds", "Couldn't load feeds from XML.", e);
			}
		}
		else printError("LoadFeeds", "Couldn't load feeds due to invalid XML document.");
	}
	else if(preferences.firstLoad.value.split("|")[1] != "1")
	{
		preferences.firstLoad.value = VERSION_NUMBER + "|1";
		savePreferences();

		// Load default feeds if no feeds are stored in memory and this is the first load
		for(var ind in DEFAULT_FEEDS)
		{
			AddFeed({url: DEFAULT_FEEDS[ind], userTitle: "", updateFrequency: DEFAULT_UPDATE_FREQUENCY, maxVisibleItems: DEFAULT_MAX_FEED_ITEMS}, true);
		}
	}
	
	RecountFeeds(true);
	
	feedsLoaded = true;
}

// Save all feeds to memory
function SaveFeeds()
{
	printFunction("SaveFeeds");

	if(widgetInitialized && feedsLoaded) // Don't save feeds if the Widget hasn't been initialized yet or the feeds hasn't been loaded
	{
		var doc = XMLDOM.createDocument();

		var feedsNode = doc.createElement("Feeds");

		// Start saving feeds
		for(var id in feedsPane.rows)
		{
			if(feedsPane.rows[id].feed && feedsPane.rows[id].feed) // Make sure there's a valid feed attached to this row
			{
				try
				{
					var feedNode = doc.createElement("Feed");
					var feed = feedsPane.rows[id].feed;

					feedNode.setAttribute("url", feed.url);
					feedNode.setAttribute("userTitle", feed.userTitle);
					feedNode.setAttribute("updateFrequency", feed.updateFrequency);
					feedNode.setAttribute("maxVisibleItems", feed.maxVisibleItems);
					feedNode.setAttribute("safeMode", feed.safeMode ? "true" : "false");

					feedsNode.appendChild(feedNode);
				}
				catch(e)
				{
					printError("SaveFeeds", "Couldn't save feed #" + id + ".", e);
				}
			}
		}

		doc.appendChild(feedsNode);

		preferences.feeds.value = doc.toXML();
		savePreferences();
	}
}

// Import feeds
function ImportFeeds()
{
	printFunction("ImportFeeds");
	
	var which = chooseFile("opml");
	
	if(which)
	{
		doc = XMLDOM.parse(filesystem.readFile(which));

		if(doc)
		{
			var outlineNodes = doc.getElementsByTagName("outline");
			
			for(var i = 0; i < outlineNodes.length; i++)
			{
				try
				{
					var outlineNode = outlineNodes.item(i);
					if(outlineNode.getAttribute("xmlUrl"))
					{
						var url = outlineNode.getAttribute("xmlUrl");
						var title = outlineNode.getAttribute("title");
						if(!title) title = outlineNode.getAttribute("text")
						
						AddFeed({url: url, userTitle: title}, true, true);
					}
				}
				catch(e)
				{
					printError("ImportFeeds", "Couldn't import feed #" + i + ".", e);
				}
			}
			
			RecountFeeds(true);
		}
	}
}

// Export current feeds
function ExportFeeds()
{
	printFunction("ExportFeeds");
	
	var where = saveAs("opml");
	
	if(where)
	{
		var doc = XMLDOM.createDocument();
		
		var opmlNode = doc.createElement("opml");
		opmlNode.setAttribute("version", "1.1");
		
		var headNode = doc.createElement("head");
		
			var titleNode = doc.createElement("title");
			titleNode.appendChild(doc.createTextNode("NewsStand Feeds"));
			
		headNode.appendChild(titleNode);
		opmlNode.appendChild(headNode);
		
		var bodyNode = doc.createElement("body");
		
		var outlineNode;
		
		for(var i = 0; i < feedsPane.rows.length; i++)
		{
			var feed = feedsPane.rows[i].feed;
			
			if(feed.type != "allFeeds")
			{
				outlineNode = doc.createElement("outline");
				outlineNode.setAttribute("text", feed.getTitle(false, true));
				outlineNode.setAttribute("type", "rss");
				outlineNode.setAttribute("title", feed.getTitle(false, true));
				outlineNode.setAttribute("xmlUrl", feed.url);
				
				bodyNode.appendChild(outlineNode);
			}
		}
		
		opmlNode.appendChild(bodyNode);
		doc.appendChild(opmlNode);
		
		filesystem.writeFile(where, doc.toXML());
	}
}

/************************************************************************
	Context Menus
*************************************************************************/

// Creates the Widget's main (default) menu
function CreateMainMenu()
{
	if(sheetMode) return;
	
	printFunction("CreateMainMenu");

	var items = new Array();
	var ind = 0;

	if(parseBool(preferences.showFeedsPane.value)) // If the feeds pane is shown
	{
		items[ind] = new MenuItem();
		items[ind].title = loc("addNewFeedMI") + "...";
		items[ind].onSelect = "focusWidget(); AddFeed(null);";

		ind++;

		if(feedsPane.rows.length > 1)
		{
			items[ind] = new MenuItem();
			items[ind].title = loc("refreshAllFeedsMI");
			items[ind].onSelect = "focusWidget(); RefreshFeeds();";

			ind++;
		}
		
		ind++;

		items[ind] = new MenuItem();
		items[ind].title = "-";
	
		ind++;
	}
	else if(feedsPane.selectedRow) // If the feeds pane is hidden and a row is selected
	{
		if (feedsPane.selectedRow != allFeeds.row) {
			items[ind] = new MenuItem();
			items[ind].title = loc("editCurrentFeedMI") + "...";
			items[ind].onSelect = "focusWidget(); EditFeed(feedsPane.selectedRow.feed);";

			ind++;
		}
		
		items[ind] = new MenuItem();
		items[ind].title = (feedsPane.selectedRow == allFeeds.row) ? loc("refreshAllFeedsMI") : loc("refreshCurrentFeedMI");
		items[ind].onSelect = "focusWidget(); onRefreshButtonClick();";

		ind++;

		items[ind] = new MenuItem();
		items[ind].title = "-";
	
		ind++;
	}

	items[ind] = new MenuItem();
	items[ind].title = "Import Feeds...";
	items[ind].onSelect = "ImportFeeds();";
	
	ind++;

	if(feedsPane.rows.length > 0)
	{
		items[ind] = new MenuItem();
		items[ind].title = "Export Feeds...";
		items[ind].onSelect = "ExportFeeds();";
	}
	
	ind++;
	
	items[ind] = new MenuItem();
	items[ind].title = "-";

	ind++;

	//items[ind] = new MenuItem();
	//items[ind].title = loc("moreAboutWidgetWorldMI") + "...";
	//items[ind].onSelect = "focusWidget(); openURL('www.WidgetWorld.com');";

	//ind++;

	//items[ind] = new MenuItem();
	//items[ind].title = loc("checkForUpdatesMI") + "...";
	//items[ind].onSelect = "focusWidget(); CheckVersion(1);";

	mainWindow.contextMenuItems = items;
}

// Return menu items related to the clicked feed
function GetFeedMenu(row)
{
	if(sheetMode) return;
	
	printFunction("GetFeedMenu");

	var items = new Array();
	var ind = 0;

	if (row != allFeeds.row) {
		
		items[ind] = new MenuItem();
		items[ind].title = loc("openLinkInBrowserMI") + "...";
		items[ind].enabled = row.feed && row.feed.link && !row.feed.refreshing;
		items[ind].onSelect = "focusWidget(); openURL(feedsPane.rows[" + row.id + "].feed.link);";

		ind++;
		
		items[ind] = new MenuItem();
		items[ind].title = loc("refreshThisFeedMI");
		items[ind].enabled = !row.feed.refreshing;
		items[ind].onSelect = "focusWidget(); feedsPane.rows[" + row.id + "].feed.refresh();";

		ind++;
		
		items[ind] = new MenuItem();
		items[ind].title = loc("editThisFeedMI") + "...";
		items[ind].onSelect = "focusWidget(); EditFeed(feedsPane.rows[" + row.id + "].feed);";

		ind++;
		
	}
	
	if(!row.feed.error && !row.feed.refreshing)
	{
		items[ind] = new MenuItem();
		items[ind].title = loc("markFeedItemsReadMI");
		items[ind].enabled = row.feed.readItems < row.feed.items.length;
		items[ind].onSelect = "focusWidget(); MarkFeedItemsAsRead(feedsPane.rows[" + row.id + "].feed, true, true);";

		ind++;
		
		items[ind] = new MenuItem();
		items[ind].title = loc("markFeedItemsUnreadMI");
		items[ind].enabled = row.feed.readItems > 0;
		items[ind].onSelect = "focusWidget(); MarkFeedItemsAsRead(feedsPane.rows[" + row.id + "].feed, false, true);";

		ind++;
	}
	
	if (row != allFeeds.row) {
		items[ind] = new MenuItem();
		items[ind].title = loc("deleteThisFeedMI");
		items[ind].onSelect = "focusWidget(); DeleteFeed(feedsPane.rows[" + row.id + "].feed, true);";

		ind++;
	}

	items[ind] = new MenuItem();
	items[ind].title = "-";

	return items;
}

// Return menu items related to the clicked item
function GetItemMenu(row)
{
	if(sheetMode) return;
	
	printFunction("GetItemMenu");

	var items = new Array();
	var ind = 0;

	items[ind] = new MenuItem();
	items[ind].title = loc("openLinkInBrowserMI");
	items[ind].enabled = (row.item && row.item.link) ? true : false;
	items[ind].onSelect = "focusWidget(); OpenItem(contentPane.rows[" + row.id + "]);";

	ind++;

	items[ind] = new MenuItem();
	items[ind].title = row.item.read ? loc("markThisItemUnreadMI") : loc("markThisItemReadMI");
	items[ind].onSelect = "focusWidget(); var item = contentPane.rows[" + row.id + "].item; item.setRead(!item.read, true); UpdateFeedTitle();";

	ind++;

	items[ind] = new MenuItem();
	items[ind].title = "-";

	return items;
}

/************************************************************************
	Sheet
*************************************************************************/

// Show the "sheet" which is passed as an argument
function ShowSheet(sheet)
{
	printFunction("ShowSheet");

	// Make sure no "sheet" is being shown currently
	if(!overlay.opacity)
	{
		// Update "Sheet frame" position and size
		sheetFrame.sheet = sheet;
		sheetFrame.hOffset = mainFrame.width/2;
		sheetFrame.width = sheet.width;
		sheetFrame.height = sheet.height;
		sheet.vOffset = -sheet.height;
		
		// Update overlay
		overlay.width = mainFrame.width - 1;
		overlay.height = mainFrame.height;

		// Open "sheet"
		moveSheetAnimation = new MoveAnimation(sheet, 0, 0, 600, animator.kEaseOut);
		
		overlay.fade.startOpacity = 0;
		overlay.fade.endOpacity = 128;

		HideTooltip(true);
		CloseTray(true);
		
		SetTooltipsEnabled(false, false);

		feedsPane.defocusRows();
		contentPane.defocusRows();
		animator.runUntilDone([moveSheetAnimation, overlay.fade]);
		
		sheetMode = true;
	}
}

// Hide the currently opened "sheet"
function HideSheet()
{
	printFunction("HideSheet");

	// Make sure there is an opened "sheet" currently
	if(overlay.opacity)
	{
		// Close "sheet"
		moveSheetAnimation = new MoveAnimation(sheetFrame.sheet, 0, -sheetFrame.sheet.height, 400, animator.kEaseIn);
		overlay.fade.startOpacity = 128;
		overlay.fade.endOpacity = 0;
		animator.runUntilDone([moveSheetAnimation, overlay.fade]);
		
		sheetFrame.sheet = null;
		sheetMode = false;
		SetTooltipsEnabled(true, preferences.descriptionsType.value == "0");
	}
}

// Used by the overlay's fade animation
function UpdateOverlayOpacity()
{
	var now = animator.milliseconds;
	var percent = Math.max(0, now - this.startTime)/this.duration;

	overlay.opacity = animator.ease(this.startOpacity, this.endOpacity, percent, animator.kEaseOut);

	if(animator.milliseconds >= this.startTime + this.duration)
	{
		overlay.opacity = this.endOpacity;
		return false;
	}

	return true;
}

/************************************************************************
	Ad Feed Managament
*************************************************************************/

// Refresh the "ad"
function RefreshAd()
{
	adFeedRequest.onreadystatechange = OnAdFeedFetchDone;
	adFeedRequest.open("GET", AD_FEED_URL, true);
	adFeedRequest.send();
}

// Called after "ad" feed is fetched
function OnAdFeedFetchDone()
{
	if(this.readyState == 4)
	{
		if(this.status == 200)
		{
			var doc = this.responseXML;

			if(doc != null)
			{
				try
				{
					var text = doc.evaluate("string(ad/text)");

					if(text != ad.data)
					{
						ad.opacity = 0;
						ad.data = text;
						ad.link = doc.evaluate("string(ad/url)");
						animator.start(ad.fadeIn);
					}
				}
				catch(e)
				{
					OnAdFeedRefreshFailed("onFeedFetchDone", "An error occurred while trying to parse data from the \"ad feed\".", e);
				}
			}
			else OnAdFeedRefreshFailed("onFeedFetchDone", "Empty XML document.");
		}
		else OnAdFeedRefreshFailed("onFeedFetchDone", "An HTTP error \"" + this.responseText + "\" occurred while trying to load the \"ad feed\".");
	}
}

// Hide "ad" if it wasn't possible to fetch its feed successfully
function OnAdFeedRefreshFailed(func, message, e)
{
	printError(func, message, e);
	ad.link = ad.data = "";
}

/************************************************************************
	Helper Functions
*************************************************************************/

// Get translated string
function loc(str)
{
	return widget.getLocalizedString(str);
}

// Return the appropriate color for an image object.
function getImageColor(inner)
{
	if(inner) return preferences.skin.value.match("fc") ? preferences.color.value : "#808080";
	else return preferences.skin.value.match("fc|bc") ? preferences.color.value : "#808080";
}

// Return the appropriate color for a text object.
function getTextColor(inner)
{
	if(inner) return preferences.skin.value.match("fc") ? (widgetLuminance ? "#000000" : "#FFFFFF") : "#FFFFFF";
	else return preferences.skin.value.match("fc|bc") ? (widgetLuminance ? "#000000" : "#FFFFFF") : "#FFFFFF";
}

// Return the appropriate shadow object for a text object.
function getShadowColor(inner)
{
	if(inner) return preferences.skin.value.match("fc") ? (widgetLuminance ? whiteShadow : blackShadow) : blackShadow;
	else return preferences.skin.value.match("fc|bc") ? (widgetLuminance ? whiteShadow : blackShadow) : blackShadow;
}

// Returns the index of an element within an array
function getArrayElementIndex(iArray, iElement)
{
	for(var i = 0; i < iArray.length; i++)
	{
		if(iArray[i] == iElement) return i;
	}

	return -1;
}

// Truncate string to a specified length
function getShortString(string, chrNum)
{
	if(string && string.length > chrNum)
	{
		string = string.substr(0, chrNum) + "...";
	}

	return string;
}

function timeToString(since, until)
{
	if(!until) until = new Date().getTime();
	var milliseconds = until - since;
	
	if(parseInt(milliseconds).toString() != "NaN" && milliseconds >= 0)
	{
		
		if(milliseconds < 60000) return loc("lessThanMinuteAgo");
		else if(milliseconds < 3600000)
		{
			if(milliseconds < 120000) return loc("aboutMinuteAgo");
			else return loc("aboutNumMinutesAgo").replace("%num%", parseInt(milliseconds/60000));
		}
		else if(milliseconds < 86400000)
		{
			if(milliseconds < 7200000) return loc("aboutHourAgo");
			else return loc("aboutNumHoursAgo").replace("%num%", parseInt(milliseconds/3600000));
		}
		else if(milliseconds < 64281600000)
		{
			if(milliseconds < 172800000) return loc("aboutDayAgo");
			else return loc("aboutNumDaysAgo").replace("%num%", parseInt(milliseconds/86400000));
		}
	}
	else return "";
}

// Returns true if the passed value can be converted to true or 1, otherwise returns false
function parseBool(val)
{
	return (val == "true" | val == "1")
}

// Make sure the 3rd argument isn't too big or too small than first two
function between(min, max, val)
{
	return Math.min(max, Math.max(min, val));
}

String.prototype.charRefToUnicode = function()
{
	return this.replace(
	/&#(([0-9]{1,7})|(x[0-9a-f]{1,6}));?/gi,
	function(match, p1, p2, p3, offset, s)
	{
	return String.fromCharCode(p2 || ("0" + p3));
	});
}

Initialize();