/************************************************************************
	Pane Object
*************************************************************************/	

function Pane(name, exType, parent)
{
	this.parent = parent;
	this.exType = exType;
	this.rows = new Array();
	this.moveRowsAnimation = new CustomAnimation(1, Pane.prototype.UpdateMoveRows);
	this.reorderable = false;
	this.dragging = false;
	
	
	// Pane's background
	this.background = new ResizableImage(name + ".background", "Resources/Images/Pane", true, parent);
	this.background.pane = this;
	this.background.onDragDrop = function(){this.pane.onDragDrop();};
	
	// Pane's overlay
	this.overlay = new Image();
	this.overlay.src = "Resources/Images/paneOverlay.png";
	parent.appendChild(this.overlay);
	
	// Pane's listWrapper
	this.listWrapper = new Frame();
	parent.appendChild(this.listWrapper);
	
	// Pane's list
	this.list = new Frame();
	this.list.pane = this;
	this.list.onMouseWheel = Pane.prototype._onMouseWheel;
	this.listWrapper.appendChild(this.list);
	
	// Pane's dragList
	this.dragList = new Frame();
	this.listWrapper.appendChild(this.dragList);
	
	// Pane's dragpreview
	this.dragPreview = new Image();
	this.dragPreview.src = "Resources/Images/dragPreview.png";
	this.dragPreview.visible = false;
	this.dragList.appendChild(this.dragPreview);
	
	// Pane's scrollbar
	this.scrollbar = new ScrollBar();
	this.scrollbar.autoHide = true;
	this.scrollbar.width = 8;
	this.scrollbar.pane = this;
	this.scrollbar.setTrackInfo(0, 0, 1, ["Resources/Images/trackTop.png", "Resources/Images/trackCenter.png", "Resources/Images/trackBottom.png"]);
	this.scrollbar.setThumbInfo(0, ["Resources/Images/thumbTop.png", "Resources/Images/thumbCenter.png", "Resources/Images/thumbBottom.png"]);
	this.scrollbar.onMouseEnter = Pane.prototype.onMouseEnterScrollbar;
	this.scrollbar.onValueChanged = Pane.prototype.onValueChangedScrollbar;
	this.list.vScrollBar = this.scrollbar;
	parent.appendChild(this.scrollbar);
	
	this.watch("hOffset", Pane.prototype.hOffsetWatch);
	this.watch("vOffset", Pane.prototype.vOffsetWatch);
	this.watch("width", Pane.prototype.widthWatch);
	this.watch("height", Pane.prototype.heightWatch);	
}

Pane.prototype.hOffsetWatch = function(prop, oldval, newval)
{
	this.hOffset = this.overlay.hOffset = this.listWrapper.hOffset = this.background.hOffset = newval;		
	return newval;
}

Pane.prototype.vOffsetWatch = function(prop, oldval, newval)
{
	this.vOffset = this.background.vOffset = newval;
	this.overlay.vOffset = this.listWrapper.vOffset = newval + 25;
	this.scrollbar.vOffset = newval + 29;
	return newval;
}

Pane.prototype.widthWatch = function(prop, oldval, newval)
{
	this.width = this.dragPreview.width = this.overlay.width = this.listWrapper.width = this.dragList.width = this.list.width = this.background.width = newval;
	this.scrollbar.hOffset = this.hOffset + newval - this.scrollbar.width - 7;
		
	for(var i = 0; i < this.rows.length; i++)
	{
		this.rows[i].width = newval;
	}
	
	return newval;
}

Pane.prototype.heightWatch = function(prop, oldval, newval)
{
	this.height = this.background.height = newval; 
	this.overlay.height = this.listWrapper.height = this.dragList.height = this.list.height = newval - 51;
	this.scrollbar.height = this.list.height - 10;
	
	return newval;
}


Pane.prototype.reorderRow = function(row, newID) {
	// Create a pointer to the row which will be selected after a row is removed
	var oldID = row.id;
	
	// Remove row
	this.rows.splice(oldID, 1);
	
	// Reinsert row
	this.rows.splice(newID, 0, row);
	
	for(var i = Math.min(oldID, newID); i < this.rows.length; i++) {
		this.rows[i].setID(i, true);
	}
	
	// Reselect the row if necessary
	if (this.selectedRow == row) {
		this.selectRow(this.selectedRow, true);
	}
	
}


Pane.prototype.addRow = function(title, id, animate)
{
	if (id == null) {
		id = this.rows.length;
	} else if(animate) {
		this.moveRowsAnimation.moveFrom = id*LIST_ROW_HEIGHT;
		this.moveRowsAnimation.moveTo = id*LIST_ROW_HEIGHT + LIST_ROW_HEIGHT;
		this.moveRowsAnimation.duration = 300;
		this.moveRowsAnimation.rows = new Array();
		for(var i = id; i < this.rows.length; i++)
		{
			this.rows[i].setID(i+1, false);
			this.moveRowsAnimation.rows[i - id] = this.rows[i];
		}
		animator.runUntilDone(this.moveRowsAnimation);
	} else {
		for(var i = id; i < this.rows.length; i++)
		{
			this.rows[i].setID(i+1, true);
		}
	}
	
	var row = new ListRow(id, title, this);
	this.rows.splice(id, 0, row);
	if(this.exType && widgetInitialized) this.selectRow(row, false);
		
	return row;
}

Pane.prototype.selectRow = function(row, triggerSelectedEvent)
{
	row = (row != null && row != undefined) ? (row instanceof ListRow ? row : this.rows[row]) : null;
	
	if(row && row == this.selectedRow) return;
	
	if(row)
	{
		if(this.selectedRow) this.selectedRow.setSelected(false);
		this.selectedRow = row;
		this.selectedRow.setSelected(true);

		if(-this.list.scrollY > this.selectedRow.vOffset)
		{
			this.dragList.scrollY = this.list.scrollY = -this.selectedRow.vOffset;
		}
		else if(this.list.height - this.list.scrollY < this.selectedRow.vOffset + LIST_ROW_HEIGHT)
		{
			this.dragList.scrollY = this.list.scrollY = Math.min(0, Math.max(this.list.height - this.rows.length*LIST_ROW_HEIGHT, this.list.height - this.selectedRow.vOffset - LIST_ROW_HEIGHT + 1));
		}		
	}
	else this.selectedRow = null;
	
	if(triggerSelectedEvent) this.onRowSelected(row);
}

Pane.prototype.updateAppearance = function()
{
	this.dragPreview.colorize = this.overlay.colorize = this.background.colorize = this.scrollbar.thumbColor = getImageColor(true);

	for(var ind in this.rows)
	{
		this.rows[ind].updateAppearance();
	}
}

Pane.prototype.removeRow = function(row, animate)
{
	var id = row;
	// Create  a pointer to the row which is about to be removed
	if (row instanceof ListRow) {
		for (var i = 0; i < this.rows.length; i++) {
			if (this.rows[i] == row) {
				id = i;
				break;
			}
		}
	}
	var removedRow = this.rows[id];
	
	// Create a pointer to the row which will be selected after a row is removed
	var selectedRow = this.selectedRow;
	
	if(removedRow == this.selectedRow) // Is the row which is about to be removed currently selected?
	{
		// Determine which row, if any, will be selected after the current one is removed
		if(id > 0) selectedRow = this.rows[id - 1]; // Select the previous row in the list, if available
		else selectedRow = this.rows.length > 1 ? this.rows[id + 1] : null; // Select the next row in the list, if available, otherwise, select none
	}
	
	// Remove row
	removedRow.remove();
	this.rows.splice(id, 1);
	
	if(removedRow == this.selectedRow) this.selectedRow = null;
	removedRow = null;
	
	if(animate && id < this.rows.length)
	{
		this.moveRowsAnimation.moveFrom = id*LIST_ROW_HEIGHT + LIST_ROW_HEIGHT;
		this.moveRowsAnimation.moveTo = id*LIST_ROW_HEIGHT;
		this.moveRowsAnimation.duration = 300;
		this.moveRowsAnimation.rows = new Array();
		for(var i = id; i < this.rows.length; i++)
		{
			this.rows[i].setID(i, false);
			this.moveRowsAnimation.rows[i - id] = this.rows[i];
		}
		animator.runUntilDone(this.moveRowsAnimation);
	}
	else
	{
		for(var i = id; i < this.rows.length; i++)
		{
			this.rows[i].setID(i, true);
		}
	}

	// Select a new row if necessary
	if(this.exType) this.selectRow(selectedRow, true);
	
	this.onRowRemoved(removedRow);
}

// Show or hide the list
Pane.prototype.setListState = function(visible, animate, synch)
{
	// Stop all animations
	if(this.list.fadeIn) this.list.fadeIn.kill();
	if(this.scrollbar.fadeIn) this.scrollbar.fadeIn.kill();
	if(this.list.fadeOut) this.list.fadeOut.kill();
	if(this.scrollbar.fadeOut) this.scrollbar.fadeOut.kill();
	
	if(animate)
	{
		if(visible)
		{
			this.list.fadeIn = new FadeAnimation(this.listWrapper, 255, 800, animator.kEaseOut);
			this.scrollbar.fadeIn = new FadeAnimation(this.scrollbar, 255, 800, animator.kEaseOut);
			this.listWrapper.opacity = this.scrollbar.opacity = 0;
			synch ? animator.runUntilDone([this.list.fadeIn, this.scrollbar.fadeIn]) : animator.start([this.list.fadeIn, this.scrollbar.fadeIn]);
			
		}
		else
		{
			this.list.fadeOut = new FadeAnimation(this.listWrapper, 0, 200, animator.kEaseIn);
			this.scrollbar.fadeOut = new FadeAnimation(this.scrollbar, 0, 200, animator.kEaseIn);
			synch ? animator.runUntilDone([this.list.fadeOut, this.scrollbar.fadeOut]) : animator.start([this.list.fadeOut, this.scrollbar.fadeOut]);
		}
	}
	else this.listWrapper.opacity = this.scrollbar.opacity = visible ? 255 : 0;
}

Pane.prototype.defocusRows = function()
{
	for(var ind in this.rows)
	{
		this.rows[ind].setSelected(this.exType && this.rows[ind] == this.selectedRow);
	}
}

// Remove all rows
Pane.prototype.clearList = function()
{
	for(var i = this.rows.length - 1; i >= 0; i--)
	{
		this.rows[i].remove();
		this.rows[i] = null;
		this.rows.splice(i, 1);
	}
}

// Used by the "move rows" animation
Pane.prototype.UpdateMoveRows = function()
{
	var now = animator.milliseconds;
	var percent = Math.max(0, now - this.startTime)/this.duration;
	
	for(var ind in this.rows)
	{
		this.rows[ind].vOffset = animator.ease(this.moveFrom + ind*LIST_ROW_HEIGHT, this.moveTo + ind*LIST_ROW_HEIGHT, percent, animator.kEaseOut);
	}
	
	if(animator.milliseconds >= this.startTime + this.duration)
	{
		for(var ind in this.rows)
		{
			this.rows.vOffset = this.moveTo + ind*LIST_ROW_HEIGHT;
		}
	
		return false;
	}
	
	return true;
}

Pane.prototype._onMouseWheel = function()
{
	var pane = this.pane;
	pane.defocusRows();
	pane.dragList.scrollY = pane.list.scrollY = Math.min(0, Math.max(-pane.rows.length*LIST_ROW_HEIGHT + this.height, this.scrollY + system.event.scrollDelta*2));
	pane.onMouseWheel();
}
Pane.prototype.onDragDrop = function(){}
Pane.prototype.onRowRemoved = function(){}
Pane.prototype.onRowSelected = function(){}
Pane.prototype.onRowContextMenu = function(){}
Pane.prototype.onRowClicked = function(){}
Pane.prototype.onMouseEnterRow = function(){}
Pane.prototype.onMouseMoveOverRow = function(){}
Pane.prototype.onMouseExitRow = function(){}
Pane.prototype.onMouseWheel = function(){}
Pane.prototype.onMouseEnterScrollbar = function(){}
Pane.prototype.onValueChangedScrollbar = function() {
	this.pane.dragList.scrollY = -this.value;
	HideTooltip();
}
