/************************************************************************
	Resizable Image Object
*************************************************************************/	

function ResizableImage(name, src, fullImage, parent)
{
	this.topLeft = new Image();
	parent.appendChild(this.topLeft);
	
	this.top = new Image();
	parent.appendChild(this.top);
	
	this.topRight = new Image();
	parent.appendChild(this.topRight);
	
	this.left = new Image();
	parent.appendChild(this.left);
	
	if(fullImage)
	{
		this.center = new Image();
		parent.appendChild(this.center);
	}
	
	this.right = new Image();
	parent.appendChild(this.right);
	
	this.bottomLeft = new Image();
	parent.appendChild(this.bottomLeft);
	
	this.bottom = new Image();
	parent.appendChild(this.bottom);
	
	this.bottomRight = new Image();
	parent.appendChild(this.bottomRight);
	
	this.watch("hOffset", ResizableImage.prototype.hOffsetWatch);
	this.watch("vOffset", ResizableImage.prototype.vOffsetWatch);
	this.watch("width", ResizableImage.prototype.widthWatch);
	this.watch("height", ResizableImage.prototype.heightWatch);	
	this.watch("src", ResizableImage.prototype.srcProperty);
	this.watch("opacity", ResizableImage.prototype.applyProperty);	
	this.watch("colorize", ResizableImage.prototype.applyProperty);	
	this.watch("onClick", ResizableImage.prototype.applyProperty);
	this.watch("onDragDrop", ResizableImage.prototype.applyProperty);
	this.watch("visible", ResizableImage.prototype.applyProperty);
	this.watch("pane", ResizableImage.prototype.applyProperty);

	this.src = src;
}

ResizableImage.prototype.hOffsetWatch = function(prop, oldval, newval)
{
	this.bottomLeft.hOffset = this.left.hOffset = this.topLeft.hOffset = newval;
	this.bottom.hOffset = this.top.hOffset = newval + this.topLeft.width;
	if(this.center) this.center.hOffset = this.bottom.hOffset;			
	return newval;
}

ResizableImage.prototype.vOffsetWatch = function(prop, oldval, newval)
{
	this.topRight.vOffset = this.top.vOffset = this.topLeft.vOffset = newval;
	this.right.vOffset = this.left.vOffset = newval + this.topLeft.height;
	if(this.center)this.center.vOffset = this.right.vOffset;
	return newval;
}

ResizableImage.prototype.widthWatch = function(prop, oldval, newval)
{
	this.bottom.width = this.top.width = newval - 2*this.topLeft.width;
	this.bottomRight.hOffset = this.right.hOffset = this.topRight.hOffset = this.top.hOffset + this.top.width;
	if(this.center) this.center.width = this.bottom.width;
	return newval;
}

ResizableImage.prototype.heightWatch = function(prop, oldval, newval)
{
	this.right.height = this.left.height = newval - 2*this.topLeft.height;
	this.bottom.vOffset = this.bottomLeft.vOffset = this.bottomRight.vOffset = this.right.vOffset + this.right.height;
	if(this.center) this.center.height = this.right.height;
	return newval;
}

ResizableImage.prototype.srcProperty = function(prop, oldval, newval)
{
	if(newval)
	{
		this.topLeft.src = newval + "/topLeft.png";
		this.top.src = newval + "/top.png";
		this.topRight.src = newval + "/topRight.png";
		this.left.src = newval + "/left.png";
		this.right.src = newval + "/right.png";
		this.bottomLeft.src = newval + "/bottomLeft.png";
		this.bottom.src = newval + "/bottom.png";
		this.bottomRight.src = newval + "/bottomRight.png";
		if(this.center) this.center.src = newval + "/center.png";
	}
	return newval;
}

ResizableImage.prototype.applyProperty = function(prop, oldval, newval)
{
	this.topLeft[prop] =
	this.top[prop] =
	this.topRight[prop] =
	this.left[prop] =
	this.right[prop] =
	this.bottomLeft[prop] =
	this.bottom[prop] =
	this.bottomRight[prop] = newval;
	if(this.center) this.center[prop] = newval;
	return newval;
}

ResizableImage.prototype.remove = function()
{
	this.topLeft.removeFromSuperview();
	this.top.removeFromSuperview();
	this.topRight.removeFromSuperview();
	this.left.removeFromSuperview();
	this.right.removeFromSuperview();
	this.bottomLeft.removeFromSuperview();
	this.bottom.removeFromSuperview();
	this.bottomRight.removeFromSuperview();
	if(this.center) this.center.removeFromSuperview();

	this.center = this.bottomRight = this.bottom = this.botttomLeft = this.right = this.left = this.top = this.topRight = this.topLeft = null;
}

ResizableImage.prototype.onDragDrop = function(){}
ResizableImage.prototype.onContextMenu = function(){}