/************************************************************************
	Tray
*************************************************************************/

var trayFrame, trayBg, trayStrip, trayResizer, trayTitle, trayDesc, trayUpdated, trayCloseTimer;

function CreateTray()
{
    if(!trayFrame)
    {
        // Tray's frame
        trayFrame = new Frame(mainWindow);
        trayFrame.vOffset = mainFrame.vOffset + 10;
        trayFrame.zOrder = -1;
        trayFrame.visible = false;
        
        // Tray's background
        trayBg = new ResizableImage("trayBg", "Resources/Images/Background", true, trayFrame);
        trayBg.hOffset = 0;
        trayBg.vOffset = 0;
        trayBg.onDragDrop = onDragDrop;
        trayBg.right.onMouseDown = "mouseX = trayFrame.width - system.event.hOffset;";
        trayBg.right.onMouseDrag = "ResizeTray(system.event.hOffset + mouseX);";
        trayBg.right.onMouseUp = "preferences.trayWidth.value = trayFrame.width;";
    
        // Tray's resizer
        trayResizer = new Image();
        trayResizer.src = "Resources/Images/resizer.png";
        trayFrame.appendChild(trayResizer);
    
        // Tray's title
        trayTitle = new Text();
        trayTitle.hOffset = 40;
        trayTitle.vOffset = 24;
        trayTitle.size = 12;
        trayTitle.wrap = true;
        trayFrame.appendChild(trayTitle);
        
        // Tray's description
        trayDesc = new Text();
        trayDesc.hOffset = 40;
        trayDesc.vOffset = 62;
        trayDesc.opacity = "70%";
        trayDesc.size = 12;
        trayDesc.wrap = true;
        trayFrame.appendChild(trayDesc);
        
        // Tray's "updated" text
        trayUpdated = new Text();
        trayUpdated.opacity = "40%";
        trayUpdated.size = 10;
        trayUpdated.hAlign = "right";
        trayUpdated.truncation = "end";
        trayFrame.appendChild(trayUpdated);
        
        // Tray close timer
        trayCloseTimer = new Timer();
        trayCloseTimer.interval = 3;
        trayCloseTimer.onTimerFired = "CloseTray(true);";
        trayCloseTimer.ticking = false;
        
        ResizeTray(preferences.trayWidth.value);
        
        trayTitle.data = "";
        trayDesc.data = "";
        trayUpdated.data = "";
    }
}

function OpenTray(row, title, description, date)
{
    if(!trayFrame) return;

    trayCloseTimer.ticking = false;
    tRow = row;

    if(trayFrame.open) UpdateTray(title, description, date);
    else
    {
        ResizeTray(preferences.trayWidth.value, mainFrame.height - 40);
        RepositionTray();
        UpdateTray(title, description, date);
        
        trayFrame.open = true;    
        trayFrame.visible = true;
     
        trayFrame.ani = new MoveAnimation(trayFrame, mainFrame.hOffset + mainFrame.width - 40, trayFrame.vOffset, 250, animator.kEaseOut);
        animator.runUntilDone(trayFrame.ani);
    }
}

function UpdateTray(title, description, date)
{
    if(!trayFrame) return;
    
    trayTitle.data = getShortString(title, 70);
    trayDesc.data = description;
    
    if(date)
    {
        trayUpdated.data = loc("updated") + " " + date;
    }
    else trayUpdated.data = "";
    
    ResizeTrayText();
}

function CloseTray(instantly)
{
    if(!trayFrame) return;
    
    trayFrame.open = false;
    tRow = null;
    
    if(instantly)
    {
	trayCloseTimer.ticking = false;
	trayFrame.ani = new MoveAnimation(trayFrame, mainFrame.hOffset + mainFrame.width - trayFrame.width - 20, trayFrame.vOffset,  250, animator.kEaseOut);
	animator.runUntilDone(trayFrame.ani);
        trayFrame.visible = false;
        ResizeTray(50, 50);
    }
    else trayCloseTimer.ticking = true;
}

function RepositionTray()
{
    if(!trayFrame) return;
    
    trayFrame.hOffset = mainFrame.hOffset + mainFrame.width - (trayFrame.open ? 40 : trayFrame.width + 20);
}

function ResizeTray(width, height)
{
    if(!trayFrame) return;
    
    if(width)
    {
        trayFrame.width = trayBg.width = between(150, MIN_BACKGROUND_WIDTH - 40, width);
        trayTitle.width = trayFrame.width - 60;
        trayDesc.width = trayFrame.width - 60;
        trayUpdated.hOffset = trayFrame.width - 20;
        trayUpdated.width = trayFrame.width - 55;
        trayResizer.hOffset = trayFrame.width - trayResizer.width/2 - 15;
    }
    
    if(height)
    {
        trayFrame.height = trayBg.height = background.height - 20;
	trayDesc.height = trayFrame.height - trayDesc.vOffset - 20;
        trayUpdated.vOffset = trayFrame.height - 19;
        trayResizer.vOffset = trayFrame.height/2  - trayResizer.height/2;
    }
    
    ResizeTrayText();
}

function ResizeTrayText()
{
    if(!trayFrame) return;
    
    trayTitle.height = -1;
    trayDesc.vOffset = trayTitle.vOffset + trayTitle.height + 10;

    if(trayUpdated.data.length > 0) trayDesc.height = trayFrame.height - trayDesc.vOffset - 33;
    else trayDesc.height = trayFrame.height - trayDesc.vOffset - 20;
}

function UpdateTrayAppearance()
{
    if(!trayFrame) return;
    
    trayBg.colorize = getImageColor();
    trayTitle.color = trayDesc.color = trayUpdated.color = getTextColor();
    trayTitle.shadow = trayDesc.shadow = trayUpdated.shadow = getShadowColor();
    trayTitle.font = trayDesc.font = trayUpdated.font = preferences.font.value;
}

function SetTrayOnWidgetFocusChange(focused)
{
    if(!trayFrame) return;
    if(!focused) trayCloseTimer.ticking = true;
}

function RemoveTray()
{
    if(!trayFrame) return;
    
    CloseTray(true);
    
    trayBg.remove();
    trayTitle.removeFromSuperview();
    trayDesc.removeFromSuperview();
    trayUpdated.removeFromSuperview();
    trayFrame.removeFromSuperview();
    
    trayFrame = trayBg = trayTitle = trayDesc  = trayUpdated = trayCloseTimer = null;
}